<?php

namespace Drp\LdapAuthentication\Http\Controllers;

use App\Http\Controllers\Controller;
use Drp\LdapAuthentication\LdapException;
use Illuminate\Contracts\Auth\Guard;
use Illuminate\Foundation\Auth\AuthenticatesUsers;
use Illuminate\Foundation\Auth\ThrottlesLogins;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

/**
 * Class LdapAuthController
 * @package Drp\LdapAuthentication\Http\Controllers
 */
class LdapAuthController extends Controller
{

    use AuthenticatesUsers, ThrottlesLogins;

    /**
     * @param Guard $auth
     */
    public function __construct(Guard $auth)
    {
        $this->auth = $auth;
        $this->middleware('guest', ['except' => 'getLogout']);
    }

    /**
     * Overriden login() method to handle custom LDAP authentication
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function login(Request $request)
    {

        $this->validate($request, [
            $this->loginUsername() => 'required',
            'password' => 'required',
        ]);

        $throttles = $this->isUsingThrottlesLoginsTrait();

        // Load credentials from user input
        $credentials = $this->getCredentials($request);

        try {

            Auth::attempt($credentials, true);

            return $this->handleUserWasAuthenticated($request, $throttles);

        } catch (LdapException $e) {

            // If the login attempt was unsuccessful we will increment the number of attempts
            // to login and redirect the user back to the login form. Of course, when this
            // user surpasses their maximum number of attempts they will get locked out.
            if ($throttles) {
                $this->incrementLoginAttempts($request);
            }

            return redirect()->back()
                ->withInput($request->only($this->loginUsername(), 'remember'))
                ->with('error', $e->getMessage());
        }

    }

    /**
     * Get the login username to be used by the controller.
     * @return string
     */
    public function loginUsername()
    {
        return property_exists($this, 'username') ? $this->username : 'username';
    }

    /**
     * Get the needed authorization credentials from the request.
     * @param  \Illuminate\Http\Request $request
     * @return array
     */
    protected function getCredentials(Request $request)
    {
        return $request->only('username', 'password');
    }

}