<?php

namespace Drp\LdapAuthentication\Providers;

use adLDAP\adLDAPException;
use adLDAP\classes\adLDAPUsers;
use Drp\LdapAuthentication\LdapException;
use Drp\LdapAuthentication\LdapConnector;
use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Contracts\Auth\UserProvider;
use Illuminate\Auth\EloquentUserProvider;
use Illuminate\Contracts\Auth\Authenticatable as UserContract;

/**
 * Class LdapUserProvider
 * @package App
 */
class LdapUserProvider extends EloquentUserProvider implements UserProvider
{

    /**
     * @var adLDAP
     */
    private $ldap;

    /**
     * The LDAP groups to allow to sign in to the CMS
     * @var string
     */
    private $ldapAllowedGroups;

    /**
     * LdapUserProvider constructor.
     * @param string $model
     * @param LdapConnector $ldap
     * @param array $ldapAllowedGroups
     */
    public function __construct(
        $model,
        LdapConnector $ldap,
        array $ldapAllowedGroups
    ) {
        $this->model = $model;
        $this->ldap = $ldap;
        $this->ldapAllowedGroups = $ldapAllowedGroups;

    }

    /**
     * Retrieve a user by the given credentials.
     * @param  array $credentials
     * @return Authenticatable|null
     * @throws LdapException
     * @throws \adLDAP\adLDAPException
     */
    public function retrieveByCredentials(array $credentials)
    {

        try {

            //Connect to the LDAP server
            $this->ldap->connect();
            // Check the credentials are valid and create an LDAP connection to use
            $authenticatedUser = $this->checkLDAPCredentials($credentials);
            // Check if the user is in the right group
            $this->checkLDAPGroups($authenticatedUser, $credentials, $this->ldapAllowedGroups);

            // Update the Laravel user model and grab a user object to complete the login
            return $this->getUser($authenticatedUser, $credentials);

        } catch (adLDAPException $e) {

            throw new LdapException($e);

        }

    }

    /**
     * Validate a user against the given credentials.
     * @param  \Illuminate\Contracts\Auth\Authenticatable $user
     * @param  array $credentials
     * @return bool
     */
    public function validateCredentials(UserContract $user, array $credentials)
    {
        //Retrieving the user has already validated them
        return true;
    }

    /**
     * Completes authentication with the LDAP service and sets up the user
     * @param array $credentials
     * @return adLDAPUsers
     * @throws LdapException
     */
    private function checkLDAPCredentials(array $credentials)
    {
        // Check if username and password exists in LDAP
        if ($this->ldap->authenticate($credentials['username'], $credentials['password'])) {
            return $this->ldap->user();
        }
        throw new LdapException('Incorrect username or password.');

    }

    /**
     * Check if the specified user is in the specified LDAP group
     * @param $authenticatedUser
     * @param array $credentials
     * @param array $groups
     * @return bool
     * @throws LdapException
     */
    private function checkLDAPGroups($authenticatedUser, array $credentials, array $groups)
    {
        //If no groups have been set, skip the check
        if (empty($this->ldapAllowedGroups)) {
            return true;
        }

        if(!method_exists($authenticatedUser, 'inGroup')){
            throw new LdapException('This Ldap connection cannot check groups.');

        }

        foreach ($groups as $group) {
            // Check with LDAP if this user has group needed on LDAP
            if ($authenticatedUser->inGroup($credentials['username'], $group)) {
                return true;
            }
        }

        throw new LdapException('You do not have permission to access this CMS.');

    }

    /**
     * Pulls info from the LDAP directory and authenticates with the local Laravel database
     * @param $ldapConnection
     * @param array $credentials
     * @return mixed
     * @throws LdapException
     */
    private function getUser($ldapConnection, array $credentials)
    {
    
        // Load the user info we want from LDAP
        $userInfo = $ldapConnection->info($credentials['username']);

        // Check we have an array of info back from LDAP
        if (!is_array($userInfo)) {
            throw new LdapException('We were unable to complete your login at this time. Please try again later.');
        }

        $user = $this->retrieveUserModel($userInfo);

        // Return the user object to be used
        return $user;

    }

    /**
     * retrieve the existing model Or create a new user record
     * @param $userInfo
     * @return mixed
     */
    private function retrieveUserModel($userInfo)
    {
               // Give the info some useful variable names
        $displayName = $userInfo[0]["displayname"][0];
        $email = $userInfo[0]["mail"][0];
        $username = $userInfo[0]["samaccountname"][0];


        $model = $this->createModel();

        // Find if they exist and load user object, or create a new User object
        $user = $model->firstOrNew(['email' => $email]);

        // Update attributes
        $user->name = $displayName;
        $user->username = $username;
        $user->save();

        return $user;

    }

}
